/*******************************************************************************
                                    ReSyEqVa                                    
                                    ========                                    

Resolve Systems of 2 Equations in 2 Variables
Copyright (C) 2010 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>

#include "main.h"
#include "utils.h"
#include "input.h"

/*******************************************************************************
 Inputs :
 * File (-f option)
 * Args (-e1 & -e2)
 * User entry (default)
*******************************************************************************/

bool args_parse(int argc, char **argv)
{
	int i, j;
	int append=0;
	bool equation_1_lock=false;
	bool equation_2_lock=false;

	char *filename=NULL;
	char *equation_1=NULL;
	char *equation_2=NULL;

	/*
	append codes :
	1=file name
	2=equation #1
	3=equation #2
	*/
	bool input_lock=false;

	for(i=1 ; i < argc ; i++)
	{
		if(argv[i][0] == '-' && append == 0)
		{
			switch(argv[i][1])
			{
				case 'f':
					if(!input_lock)
						append=1;
					else
						printf("Error: you can only specify the input once (next values are ignored).\n");
				break;
				case 'e':
					switch(argv[i][2])
					{
						case '1':
							if(!input_lock && !equation_1_lock)
							{
								append=2;
								equation_1_lock=true;
							}
							else
								printf("Error: you can only specify the input once (next values are ignored).\n");
						break;
						case '2':
							if(!input_lock && !equation_2_lock)
							{
								append=3;
								equation_2_lock=true;
							}
							else
								printf("Error: you can only specify the input once (next values are ignored).\n");
						break;
						default:
							printf("Unknown argument: \"%s\".\n", argv[i]);
						break;
					}
				break;
				case '-':
					if(!strcmp(argv[i],"--help"))
					{
						printf("%s %s %s\n\n%s\n", SOFTWARE, VERSION, COPYRIGHT, HELP);
						exit(0);
					}
					else if(!strcmp(argv[i],"--version"))
					{
						printf("%s %s %s\n", SOFTWARE, VERSION, COPYRIGHT);
						exit(0);
					}
					else
						printf("Unknown argument: \"%s\".\n", argv[i]);
				break;
				default:
					printf("Unknown argument: \"%s\".\n", argv[i]);
				break;
			}
		}
		else
		{
			if(append == 1)
			{
				filename=malloc(sizeof(char) * sizeof(argv[i]));
				strcpy(filename, argv[i]);

				append=0;
				input_lock=true;
			}
			else if(append == 2)
			{
				equation_1=malloc(sizeof(char) * sizeof(argv[i]));
				strcpy(equation_1, argv[i]);

				append=0;
			}
			else if(append == 3)
			{
				equation_2=malloc(sizeof(char) * sizeof(argv[i]));
				strcpy(equation_2, argv[i]);

				append=0;
			}
			else
				printf("Unknown value: \"%s\".\n", argv[i]);

			if(equation_1_lock && equation_2_lock)
				input_lock=true;
		}
	}

	if((equation_1_lock && !equation_2_lock) || (!equation_1_lock && equation_2_lock))
		printf("Error: you must specify the values of the both equations !");

	if(input_lock)
	{
		if(filename != NULL)
			input_file_parse(filename);
		else if(equation_1 != NULL && equation_2 != NULL)
			input_equations_parse(equation_1, equation_2);


		free(filename);
		free(equation_1);
		free(equation_2);
		return true;
	}
	else
	{
		free(filename);
		free(equation_1);
		free(equation_2);
		return false;
	}
}

void input_parse(char *equations)
{
	int read_state=1;
	int queue_int=0;
	int queue_operation=0;
	int queue_reverse=false;
	int i=0;
	char equations_char;

        for(i=0 ; 1 ; i++)
        {
		equations_char=equations[i];

		if(equations_char > 47 && equations_char < 58)
		{
			if(!queue_int)
				queue_int=equations_char-48;
			else
				queue_int=queue_int*10 + equations_char-48;
		}
		else
		{	
			if(queue_operation > 10 && queue_int)
			{
				if(queue_reverse)
				{
					queue_int=queue_int * -1;
					queue_reverse=false;
				}

				switch(queue_operation)
				{
					case 11:
						if(read_state == 1)
							equa_1_x_fact=queue_int;
						else 
							equa_2_x_fact=queue_int;
					break;
					case 22:
						if(read_state == 1)
							equa_1_y_fact=queue_int;
						else 
							equa_2_y_fact=queue_int;
					break;
					case 33:
						if(read_state == 1)
							equa_1_result=queue_int;
						else 
							equa_2_result=queue_int;
					break;
				}

				queue_operation=0;
				queue_int=0;
			}

			switch(equations_char)
			{
				case 'x':
					if(queue_int)
					{
						if(queue_reverse)
						{
							queue_int=queue_int * -1;
							queue_reverse=false;
						}

						if(read_state == 1)
							equa_1_x_fact=queue_int;
						else 
							equa_2_x_fact=queue_int;

						queue_int=0;
					}
					else
						queue_operation=1;
				break;
				case 'y':
					if(queue_int)
					{
						if(queue_reverse)
						{
							queue_int=queue_int * -1;
							queue_reverse=false;
						}

						if(read_state == 1)
							equa_1_y_fact=queue_int;
						else 
							equa_2_y_fact=queue_int;

						queue_int=0;
					}
					else
						queue_operation=2;
				break;
				case '*':
					if(queue_operation)
						queue_operation=queue_operation*11;
				break;
				case '-':
					queue_reverse=1;

					if(queue_operation == 1)
					{
						if(read_state == 1)
							equa_1_x_fact=1;
						else 
							equa_2_x_fact=1;

						if(queue_reverse)
						{
							if(read_state == 1)
								equa_1_x_fact=equa_1_x_fact*-1;
							else 
								equa_2_x_fact=equa_2_x_fact*-1;

							queue_reverse=false;
						}

						queue_int=0;
						queue_operation=0;
					}
				case '+':
					if(queue_operation == 1)
					{
						if(read_state == 1)
							equa_1_x_fact=1;
						else 
							equa_2_x_fact=1;

						if(queue_reverse)
						{
							if(read_state == 1)
								equa_1_x_fact=equa_1_x_fact*-1;
							else 
								equa_2_x_fact=equa_2_x_fact*-1;

							queue_reverse=false;
						}

						queue_int=0;
						queue_operation=0;
					}
				break;
				break;
				case '=':
					if(queue_operation == 2)
					{
						if(read_state == 1)
							equa_1_y_fact=1;
						else 
							equa_2_y_fact=1;

						if(queue_reverse)
						{
							if(read_state == 1)
								equa_1_y_fact=equa_1_y_fact*-1;
							else 
								equa_2_y_fact=equa_2_y_fact*-1;

							queue_reverse=false;
						}
					}

					queue_int=0;
					queue_operation=33;
				break;
			}
		}

		if(equations_char == '\n')
			read_state++;

		if(read_state > 2)
			break;

		if(equations_char == '\0')
			break;
        }
}

void input_file_parse(char *input_file_name)
{
	FILE *input_file_object=NULL;
	char input_file_char;
	char *equations=NULL;	
	int i;

	input_file_object=fopen(input_file_name, "r");

	equations=malloc(sizeof(char) * file_length(input_file_object));

	if(!input_file_object)
	{
		printf("Error while opening file : %s.\n", input_file_name);
		exit(0);
	}

	for(i=0 ; 1 ; i++)
	{
		input_file_char=fgetc(input_file_object);

		equations[i]=input_file_char;
	
		if(input_file_char == EOF)
		{
			equations[i]='\0';
			break;
		}
		else
			equations[i]=input_file_char;
	}

	fclose(input_file_object);

	input_parse(equations);

	free(equations);
}

void input_equations_parse(char *equation_1, char *equation_2)
{
	char *equations=NULL;
	equations=malloc(sizeof(char) * sizeof(equation_1) + sizeof(char) * sizeof(equation_2));

	sprintf(equations, "%s\n%s", equation_1, equation_2);

	input_parse(equations);

	free(equations);
}
